# plugin_info, a plugin for working with information about plugins.
# Copyright (C) 2008, 2010 Canonical Limited.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
# 

"""Extraction of plugin data from a plugin.

The primary entry point to this module is extract_info_url which will attempt
to do the right thing when given a url (including directory service lookups).
"""

__all__ = [
    'extract_info_branch',
    'extract_info_local',
    'extract_info_url',
    'PluginInfo',
    ]

import sys

from bzrlib import branch, directory_service, osutils


def extract_info_branch(branch):
    """Extract plugin info from a branch containing a plugin.

    :param branch: The branch containing the plugin.
    :return: A PluginInfo for the plugin.
    """
    tempdir = osutils.mkdtemp()
    treedir = tempdir + '/plugin'
    try:
        branch.create_checkout(treedir, lightweight=True)
        result = extract_info_local(treedir)
        result.location = branch.base
        return result
    finally:
        osutils.rmtree(tempdir, ignore_errors=True)


def extract_info_local(path):
    """Extract plugin info from a plugin directory on disk.

    :param path: The path the plugin can be found at.
    :return: A PluginInfo for the plugin at path.
    """
    # setup a fake 'package' that this setup can import siblings from
    package = MyPackage(path)
    source_path = path  + '/setup.py'
    source_file = open(source_path, 'rb')
    try:
        source = source_file.read()
    finally:
        source_file.close()
    code = compile(source, 'setup.py', 'exec')
    eval_globals = {'__file__':source_path,
        '__name__':'setup',
        '__package__':package.__name__}
    sys.modules[package.__name__] = package
    try:
        module = eval(code, eval_globals)
    finally:
        del sys.modules[package.__name__]
        to_unload = []
        for module in sys.modules.keys():
            if module.startswith(package.__name__ + '.'):
                to_unload.append(module)
        for module in to_unload:
            del sys.modules[module]
    result = PluginInfo()
    result.name = eval_globals.get('bzr_plugin_name', None)
    result.commands = eval_globals.get('bzr_commands', [])
    result.version_info = eval_globals.get('bzr_plugin_version', None)
    result.minimum_bzr_version = eval_globals.get('bzr_minimum_version', None)
    result.maximum_bzr_version = eval_globals.get('bzr_maximum_version', None)
    result.control_formats = eval_globals.get('bzr_control_formats', {})
    result.checkout_formats = eval_globals.get('bzr_checkout_formats', {})
    result.branch_formats = eval_globals.get('bzr_branch_formats', {})
    result.repository_formats = eval_globals.get('bzr_repository_formats', {})
    result.transports = eval_globals.get('bzr_transports', [])
    result.location = path
    return result


def extract_info_url(url):
    """Extract info from a plugin at url.

    :param url: A URL to probe. Directory service lookups will be done on it.
    :return: A PluginInfo for the plugin at url.
    """
    # TODO: for local branches use extract_info_local ?:- what if they are
    # treeless branches?
    b = branch.Branch.open(url)
    result = extract_info_branch(b)
    result.location = url
    return result


class PluginInfo(object):
    """Information about a plugin."""

    def __init__(self):
        """Create a PluginInfo instance."""
        self.name = None
        self.location = None
        self.commands = []
        self.version_info = None
        self.minimum_bzr_version = None
        self.maximum_bzr_version = None
        self.control_formats = {}
        self.checkout_formats = {}
        self.branch_formats = {}
        self.repository_formats = {}
        self.transports = []

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __str__(self):
        version_string = '%s.%s.%s%s%s' % self.version_info
        command_string = ','.join('"%s"' % command for command in self.commands)
        control_string = ','.join('"%s"' % format for format in self.control_formats.keys())
        checkout_string = ','.join('"%s"'% format for format in self.checkout_formats.values())
        branch_string = ','.join('"%s"'% format for format in self.branch_formats.values())
        repository_string = ','.join('"%s"'% format for format in self.repository_formats.values())
        transports_string = ','.join('"%s"' % prefix for prefix in self.transports)
        return ('%s(%s) from %s supplies commands(%s), control formats(%s), '
            'checkout formats(%s), branch formats(%s), repository formats(%s), '
            'transports(%s).'
            % (self.name, version_string, self.location, command_string, control_string,
               checkout_string, branch_string, repository_string, transports_string))

    def to_python(self):
        """Represent this PluginInfo as a python source fragment."""
        result = []
        result.append('plugin = PluginInfo()\n')
        result.append('plugin.name = %r\n' % (self.name,))
        result.append('plugin.location = %r\n' % (self.location,))
        result.append('plugin.version_info = %s\n' % (self.version_info,))
        result.append('plugin.minimum_bzr_version = %s\n' % (self.minimum_bzr_version,))
        result.append('plugin.maximum_bzr_version = %s\n' % (self.maximum_bzr_version,))
        result.append('plugin.commands = %s\n' % (self.commands,))
        result.append('plugin.control_formats = %s\n' % (self.control_formats,))
        result.append('plugin.checkout_formats = %s\n' % (self.checkout_formats,))
        result.append('plugin.branch_formats = %s\n' % (self.branch_formats,))
        result.append('plugin.repository_formats = %s\n' % (self.repository_formats,))
        result.append('plugin.transports = %s\n' % (self.transports,))
        return ''.join(result)


class MyPackage(object):
    """A package to isolate (a little) imports."""

    def __init__(self, dir):
        setattr(self, '__file__', dir + '/__init__.py')
        setattr(self, '__name__', 'setup_plugin_info')
        setattr(self, '__package__', 'setup_plugin_info')
        setattr(self, '__path__', [dir])
