# plugin_info, a plugin for working with information about plugins.
# Copyright (C) 2008 Canonical Limited.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
# 

"""Tests for extracting data from a plugin."""

from copy import copy
import sys

from bzrlib import directory_service
from bzrlib.plugins.plugin_info.extract import *
from bzrlib.tests import TestCaseWithTransport


class TestPluginInfo(TestCaseWithTransport):

    def sample_plugin(self):
        plugin = PluginInfo()
        plugin.name = "plugin_info"
        plugin.commands = ["foo"]
        plugin.version_info = (1, 2, 0, 'dev', 0)
        plugin.minimum_bzr_version = (1, 0, 0)
        plugin.maximum_bzr_version = None
        plugin.control_formats = {"svn":{"ignored":None}}
        plugin.checkout_formats = {"signature":"dirstate"}
        plugin.branch_formats = {"signature":"format 6"}
        plugin.repository_formats = {"signature":"packs"}
        plugin.transports = ["hg+ssh://"]
        plugin.location = "http://bazaar-vcs.org/plugin-info/"
        return plugin

    def test_construct(self):
        plugin = PluginInfo()

    def test_defaults(self):
        plugin = PluginInfo()
        self.assertEqual(None, plugin.name)
        self.assertEqual(None, plugin.location)
        self.assertEqual([], plugin.commands)
        self.assertEqual(None, plugin.version_info)
        self.assertEqual(None, plugin.minimum_bzr_version)
        self.assertEqual(None, plugin.maximum_bzr_version)
        self.assertEqual({}, plugin.control_formats)
        self.assertEqual({}, plugin.checkout_formats)
        self.assertEqual({}, plugin.branch_formats)
        self.assertEqual({}, plugin.repository_formats)

    def test_str(self):
        plugin = self.sample_plugin()
        self.assertEqual('plugin_info(1.2.0dev0) from http://bazaar-vcs.org/plugin-info/ supplies commands("foo"), control formats("svn"), checkout formats("dirstate"), branch formats("format 6"), repository formats("packs"), transports("hg+ssh://").',
            str(plugin))

    def test_python(self):
        # A plugin info can be turned into a python source fragment.
        plugin = self.sample_plugin()
        self.assertEqual("""plugin = PluginInfo()
plugin.name = 'plugin_info'
plugin.location = 'http://bazaar-vcs.org/plugin-info/'
plugin.version_info = (1, 2, 0, 'dev', 0)
plugin.minimum_bzr_version = (1, 0, 0)
plugin.maximum_bzr_version = None
plugin.commands = ['foo']
plugin.control_formats = {'svn': {'ignored': None}}
plugin.checkout_formats = {'signature': 'dirstate'}
plugin.branch_formats = {'signature': 'format 6'}
plugin.repository_formats = {'signature': 'packs'}
plugin.transports = ['hg+ssh://']
""", plugin.to_python())

    def test___eq__(self):
        plugin_expected = PluginInfo()
        plugin_expected.name = "1"
        plugin_expected.commands = "2"
        plugin_expected.version_info = "3"
        plugin_expected.minimum_bzr_version = "4"
        plugin_expected.maximum_bzr_version = "5"
        plugin_expected.control_formats = "6"
        plugin_expected.checkout_formats = "7"
        plugin_expected.branch_formats = "8"
        plugin_expected.repository_formats = "9"
        plugin_expected.location = "10"
        plugin = copy(plugin_expected)
        self.assertEqual(plugin_expected, plugin)
        plugin.name = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.commands = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.version_info = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.minimum_bzr_version = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.maximum_bzr_version = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.control_formats = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.checkout_formats = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.branch_formats = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.repository_formats = None
        self.assertNotEqual(plugin_expected, plugin)

        plugin = copy(plugin_expected)
        plugin.location = None
        self.assertNotEqual(plugin_expected, plugin)


class TestExtraction(TestCaseWithTransport):

    def test_extract_no_variables(self):
        self.build_tree(['foo-bar/'])
        self.build_tree_contents([('foo-bar/setup.py', '')])
        plugin = extract_info_local('foo-bar')
        expected_plugin = PluginInfo()
        expected_plugin.location = 'foo-bar'
        self.assertEqual(expected_plugin, plugin)

    def sample_setup(self):
        return """
bzr_plugin_name = "foo_bar"
bzr_commands = ["a-command"]
bzr_plugin_version = (1, 2, 3, 'dev', 0)
bzr_minimum_version = (1, 2, 0)
bzr_maximum_version = (1, 3, 0)
bzr_control_formats = {"bzr-meta":{".bzr/branch-format":"Bazaar-NG meta directory, format 1\\n"}}
bzr_checkout_formats = {"Bazaar Working Tree Format 4 (bzr 0.15)\\n":"dirstate"}
bzr_branch_formats = {"Bazaar Branch Format 6 (bzr 0.15)\\n":"branch 6"}
bzr_repository_formats = {"Bazaar pack repository format 1 (needs bzr 0.92)\\n":"packs"}
bzr_transports = ["hg+ssh://"]
"""

    def setup_plugin(self):
        tree = self.make_branch_and_tree('plugin')
        self.build_tree_contents([('plugin/setup.py', self.sample_setup())])
        tree.add(['setup.py'])
        tree.commit('yay history')
        return tree

    def expected_plugin(self):
        expected_plugin = PluginInfo()
        expected_plugin.name = "foo_bar"
        expected_plugin.commands = ["a-command"]
        expected_plugin.version_info = (1, 2, 3, 'dev', 0)
        expected_plugin.minimum_bzr_version = (1, 2, 0)
        expected_plugin.maximum_bzr_version = (1, 3, 0)
        expected_plugin.control_formats = {
            "bzr-meta":{
                ".bzr/branch-format":"Bazaar-NG meta directory, format 1\n"}}
        expected_plugin.checkout_formats = {
            "Bazaar Working Tree Format 4 (bzr 0.15)\n":"dirstate"}
        expected_plugin.branch_formats = {
            "Bazaar Branch Format 6 (bzr 0.15)\n":"branch 6"}
        expected_plugin.repository_formats = {
            "Bazaar pack repository format 1 (needs bzr 0.92)\n":"packs"}
        expected_plugin.transports = ["hg+ssh://"]
        return expected_plugin

    def test_extract_all_variables(self):
        self.build_tree(['foo--bar/'])
        self.build_tree_contents([('foo--bar/setup.py', self.sample_setup())])
        plugin = extract_info_local('foo--bar')
        expected_plugin = self.expected_plugin()
        expected_plugin.location = 'foo--bar'
        self.assertEqual(expected_plugin, plugin)

    def test_extract_imports_sibling(self):
        self.build_tree(['foo--bar/'])
        self.build_tree_contents([('foo--bar/info.py', self.sample_setup()),
            ('foo--bar/setup.py', 'from info import *\n')])
        imported = list(sys.modules.keys())
        plugin = extract_info_local('foo--bar')
        expected_plugin = self.expected_plugin()
        expected_plugin.location = 'foo--bar'
        self.assertEqual(expected_plugin, plugin)
        self.assertEqual(imported, sys.modules.keys())

    def test_extract_info_branch(self):
        tree = self.setup_plugin()
        expected_plugin = self.expected_plugin()
        expected_plugin.location = tree.branch.base
        self.assertEqual(expected_plugin, extract_info_branch(tree.branch))

    def test_extract_info_url_handles_directory_lookup(self):
        self.setup_plugin()
        class LocalDirectory:
            def look_up(self, name, url):
                return 'plugin'
        directory_service.directories.register('pl:', LocalDirectory, '')
        def unregister():
            d = directory_service.directories
            d.remove('pl:')
            del d._help_dict['pl:']
            del d._info_dict['pl:']
        self.addCleanup(unregister)
        expected_plugin = self.expected_plugin()
        expected_plugin.location = 'pl:'
        self.assertEqual(expected_plugin, extract_info_url('pl:'))
